--------------------------------------------------------
-- Archivo creado  - mircoles-enero-02-2013   
--------------------------------------------------------
--------------------------------------------------------
--  DDL for Package PACK_CONSULTES
--------------------------------------------------------

  CREATE OR REPLACE PACKAGE "USER1"."PACK_CONSULTES" AS 

TYPE tcursor IS REF CURSOR;

PROCEDURE PRC_CONSULTES_A( IDPAIS IN CHAR,CURSOR_A OUT tcursor, RSP OUT VARCHAR2);
PROCEDURE PRC_CONSULTES_B(CURSOR_B OUT tcursor, RSP OUT VARCHAR2);
PROCEDURE PRC_CONSULTES_C(IDAPLIC IN NUMBER, ANYY IN NUMBER,CURSOR_C OUT tcursor, RSP OUT VARCHAR2);
PROCEDURE PRC_CONSULTES_D(MOBILUSR IN CHAR, CURSOR_D OUT tcursor, RSP OUT VARCHAR2);
PROCEDURE PRC_CONSULTES_E(ANYY IN NUMBER,CURSOR_E OUT tcursor, RSP OUT VARCHAR2);

PROCEDURE PRC_PAISOS(CURSOR_PAISOS OUT tcursor, RSP OUT VARCHAR2);

END PACK_CONSULTES;

/
--------------------------------------------------------
--  DDL for Package Body PACK_CONSULTES
--------------------------------------------------------

  CREATE OR REPLACE PACKAGE BODY "USER1"."PACK_CONSULTES" AS

/*
Aquest primer procediment realitza una consulta sobre una vista preparada amb les dades que volem "el llistat de tots els
desenvolupadors d'un pas donat amb totes les seves dades, incloent el nmero d'aplicacions diferents publicades
*/
PROCEDURE PRC_CONSULTES_A
(
IDPAIS IN CHAR
,CURSOR_A OUT tcursor
, RSP OUT VARCHAR2
) 
IS
TS TIMESTAMP;
DATADES TIMESTAMP;
APLIC NUMBER;
BEGIN
TS:=SYSTIMESTAMP;


OPEN CURSOR_A FOR 
SELECT * FROM VIEW_CONSULTA_A WHERE ID_PAIS=IDPAIS; /*La vista ja est preparada amb les dades i noms s'ha de filtrar per pas*/

RSP:='OK'; /*Si sot ha anat b ho notifiquem*/
/*Inserim en el log de procediments el resultat de l'execuci del procediment*/
INSERT INTO log_procediments (ID_PRC,DATACRIDA,PENT1,PARAM_SORT) VALUES ('PRC_CONSULTES_A',TS,IDPAIS,RSP);

/*En cas d'alguna excepci no controlada*/
EXCEPTION
  WHEN OTHERS THEN
  RSP:=SUBSTR(sqlerrm,1,200);
INSERT INTO log_procediments (ID_PRC,DATACRIDA,PENT1,PARAM_SORT) VALUES ('PRC_CONSULTES_A',TS,IDPAIS,RSP);
END PRC_CONSULTES_A;

/*
Aquest procediment ens permet, via vista preparada VIEW_CONSULTA_B "el llistat de totes les aplicacions actives i de les 
seves dades principals, ordenat pel nmero total de descrregues que han tingut fins al moment a nivell mundial.
*/
PROCEDURE PRC_CONSULTES_B 
(
CURSOR_B OUT tcursor
, RSP OUT VARCHAR2
)
IS
TS TIMESTAMP;
BEGIN
TS:=SYSTIMESTAMP;
/*obrim el cursor que podrem consultar*/
OPEN CURSOR_B FOR
SELECT * FROM VIEW_CONSULTA_B ORDER BY NUMDESC DESC;

RSP:='OK'; /*Si sot ha anat b ho notifiquem*/
/*Inserim en el log de procediments el resultat de l'execuci del procediment*/
INSERT INTO log_procediments (ID_PRC,DATACRIDA,PARAM_SORT) VALUES ('PRC_CONSULTES_B',TS,RSP);

/*En cas d'alguna excepci no controlada*/
EXCEPTION
  WHEN OTHERS THEN
  RSP:=SUBSTR(sqlerrm,1,200);
INSERT INTO log_procediments (ID_PRC,DATACRIDA,PARAM_SORT) VALUES ('PRC_CONSULTES_B',TS,RSP);
END PRC_CONSULTES_B;


/*
Aquest procediment permet "donada una aplicaci i un any concret, el llistat de tots els pasos on s'ha descarregat
aquell any, aix com el nmero de descrregues que ha tingut a cada pas.
*/
PROCEDURE PRC_CONSULTES_C 
(
IDAPLIC IN NUMBER
, ANYY IN NUMBER
, CURSOR_C OUT tcursor
, RSP OUT VARCHAR2
) 
IS
TS TIMESTAMP;
BEGIN
TS:=SYSTIMESTAMP;

/*obrim el cursor amb les dades que necessitarem*/
OPEN CURSOR_C FOR
SELECT P.ID_PAIS, P.NOM_PAIS,ANYCONCRET, TOTALDESCPAIS FROM PAISOS P 
JOIN (SELECT COUNT(*) AS TOTALDESCPAIS,EXTRACT(YEAR FROM DATADESCARREGA) AS ANYCONCRET, ID_PAIS FROM DESCARREGUES WHERE EXTRACT(YEAR FROM DATADESCARREGA)=ANYY AND ID_APLICACIO=IDAPLIC GROUP BY ID_PAIS
,EXTRACT(YEAR FROM DATADESCARREGA)) DESCA ON P.ID_PAIS=DESCA.ID_PAIS;

RSP:='OK';/*Si sot ha anat b ho notifiquem*/

/*Inserim en el log de procediments el resultat de l'execuci del procediment*/
INSERT INTO log_procediments (ID_PRC,DATACRIDA,PENT1,PENT2,PARAM_SORT) VALUES ('PRC_CONSULTES_C',TS,TO_CHAR(IDAPLIC),
TO_CHAR(ANYY),RSP);

/*En cas d'alguna excepci no controlada*/
EXCEPTION
  WHEN OTHERS THEN
  RSP:=SUBSTR(sqlerrm,1,200);
INSERT INTO log_procediments (ID_PRC,DATACRIDA,PENT1,PENT2,PARAM_SORT) VALUES ('PRC_CONSULTES_C',TS,TO_CHAR(IDAPLIC),
TO_CHAR(ANYY),RSP);
END PRC_CONSULTES_C;

/*
Procediment que permet "Donat un usuari final (identificat pel seu nmero de telfon), el llistat de tota la seva activitat
de descrregues a la plataforma, incloent data, aplicaci descarregada, preu que va pagar, etc...
*/
PROCEDURE PRC_CONSULTES_D 
(
MOBILUSR IN CHAR
, CURSOR_D OUT tcursor
, RSP OUT VARCHAR2
) 
IS
TS TIMESTAMP;
BEGIN
TS:=SYSTIMESTAMP;

/*obrim el cursor amb les dades que necessitarem*/
OPEN CURSOR_D FOR
SELECT AP.NOM,D.ID_DESCARREGA,D.ID_USUARI,D.ID_APLICACIO,D.DATADESCARREGA,
D.PREUPAGAT,D.ID_FP,D.IMEI,D.ID_OPERADOR,D.ID_PAIS,D.VERSIO,D.ID_DIVISA 
FROM DESCARREGUES D JOIN USUARIS U ON D.ID_USUARI=U.ID_USUARI JOIN APLICACIONS AP ON
  D.ID_APLICACIO=AP.ID_APLICACIO WHERE MOBIL=MOBILUSR;


RSP:='OK';/*Si sot ha anat b ho notifiquem*/

/*Inserim en el log de procediments el resultat de l'execuci del procediment*/
INSERT INTO log_procediments (ID_PRC,DATACRIDA,PENT1,PARAM_SORT) VALUES ('PRC_CONSULTES_D',TS,MOBILUSR,RSP);

/*En cas d'alguna excepci no controlada*/
EXCEPTION
  WHEN OTHERS THEN
  RSP:=SUBSTR(sqlerrm,1,200);
INSERT INTO log_procediments (ID_PRC,DATACRIDA,PENT1,PARAM_SORT) VALUES ('PRC_CONSULTES_D',TS,MOBILUSR,RSP);
END PRC_CONSULTES_D;

/*
Aquest procediment permet "Donat un any concret, el llistat dels 20 usuaris que ms diners s'han gastat en aplicacions
mbils, ordenat de ms a menys.
*/
PROCEDURE PRC_CONSULTES_E
(
ANYY IN NUMBER
, CURSOR_E OUT tcursor
, RSP OUT VARCHAR2
)
IS
TS TIMESTAMP;
BEGIN
TS:=SYSTIMESTAMP;

/*obrim el cursor amb les dades que necessitarem*/
OPEN CURSOR_E FOR
SELECT U.ID_USUARI,U.NOM,U.COGNOM1,U.MOBIL,TOTALGASTAT FROM USUARIS U JOIN (SELECT ID_USUARI,SUM(PREUPAGAT) AS TOTALGASTAT 
  FROM DESCARREGUES WHERE EXTRACT(YEAR FROM DATADESCARREGA)=ANYY GROUP BY ID_USUARI) D ON U.ID_USUARI=D.ID_USUARI 
  WHERE ROWNUM<21 ORDER BY TOTALGASTAT DESC ;

RSP:='OK';/*Si sot ha anat b ho notifiquem*/

/*Inserim en el log de procediments el resultat de l'execuci del procediment*/
INSERT INTO log_procediments (ID_PRC,DATACRIDA,PENT1,PARAM_SORT) VALUES ('PRC_CONSULTES_E',TS,ANYY,RSP);
/*En cas d'alguna excepci no controlada*/
EXCEPTION
  WHEN OTHERS THEN
  RSP:=SUBSTR(sqlerrm,1,200);
INSERT INTO log_procediments (ID_PRC,DATACRIDA,PENT1,PARAM_SORT) VALUES ('PRC_CONSULTES_E',TS,ANYY,RSP);
END PRC_CONSULTES_E;


/*Aquest procediment el creem bsicament per facilitar les consultes de pasos en l'aplicaci de proves*/
PROCEDURE PRC_PAISOS 
(
CURSOR_PAISOS OUT tcursor
, RSP OUT VARCHAR2
) 
IS
TS TIMESTAMP;
BEGIN
TS:=SYSTIMESTAMP;
/*obrim el cursor amb les dades que necessitarem*/
OPEN CURSOR_PAISOS FOR
SELECT * FROM PAISOS WHERE ID_PAIS IN (SELECT ID_PAIS FROM DESENVOLUPADORS GROUP BY ID_PAIS);

RSP:='OK';/*Si sot ha anat b ho notifiquem*/
/*Inserim en el log de procediments el resultat de l'execuci del procediment*/
INSERT INTO log_procediments (ID_PRC,DATACRIDA,PARAM_SORT) VALUES ('PRC_PAISOS',TS,RSP);
/*En cas d'alguna excepci no controlada*/
EXCEPTION
  WHEN OTHERS THEN
  RSP:=SUBSTR(sqlerrm,1,200);
INSERT INTO log_procediments (ID_PRC,DATACRIDA,PARAM_SORT) VALUES ('PRC_PAISOS',TS,RSP);
END PRC_PAISOS;

END PACK_CONSULTES;

/
--------------------------------------------------------
--  DDL for Procedure BXA_APLICACIONS
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "USER1"."BXA_APLICACIONS" 
/*
Procediment del conjunt ABM de les aplicacions. Amb aquesta procediment es dna de baixa una
aplicaci del sistema, marcant-la, donant un valor al camp "databaixa".
*/
(
  IDE IN NUMBER /* variable entrada identificador de l'aplicaci */
 ,RSP OUT VARCHAR2 /*variable informativa de sortida*/
)
AS 
TS TIMESTAMP; /*variable amb la data actual del sistema */
BEGIN
 TS:=SYSTIMESTAMP; /*variable que ens guarda el valor de la data del sistema */
  /* Actualitzem la taula aplicacions, donant de baixa l'aplicaci posant una data al camp
  databaixa. */
  UPDATE APLICACIONS SET databaixa=TS,activa=0 WHERE id_aplicacio=IDE;
  /*Hem actualitzat i ha anat tot b. ho registrem a la variable de sortida*/
  RSP:='OK';
  INSERT INTO LOG_PROCEDIMENTS (ID_PRC,DATACRIDA,PENT1,PARAM_SORT)
  VALUES ('BXA_APLICACIONS',TS,TO_CHAR(IDE),RSP);
  /*tanquem la transacci mitjanant commit*/  
  COMMIT;
  /*En cas d'una inesperada excepci*/
  EXCEPTION
  WHEN OTHERS THEN
  /* Desfem tot el que hi havia executat*/
  ROLLBACK;
  /* Informem de l'error a la variable de sortida i guardem en el log de procediments*/
  RSP:=SUBSTR(sqlerrm,1,200);
  INSERT INTO LOG_PROCEDIMENTS (ID_PRC,DATACRIDA,PENT1,PARAM_SORT)
  VALUES ('BXA_APLICACIONS',TS,TO_CHAR(IDE),RSP);
  /*confirmem les operacions*/  
  COMMIT;
  
  NULL;
END BXA_APLICACIONS;

/
--------------------------------------------------------
--  DDL for Procedure BXA_DVR
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "USER1"."BXA_DVR" 
/*
Procediment del conjunt ABM de desenvolupadors. Aquest procediment dna de baixa del sistema
al desenvolupador indicat. Inserint en el camp "databaixa" de la taula DESENVOLUPADORS.
*/
(
  IDE IN NUMBER /*variable d'entrada. Identificador del desenvolupador*/
 ,RSP OUT VARCHAR2 /*variable de sortida informativa*/
)AS 
TS TIMESTAMP;
BEGIN
  TS:=SYSTIMESTAMP; /*guardem la data del sistema que ens servir per tot el procediment*/
  /*Actualitzem la taula DESENVOLUPADORS */
  UPDATE desenvolupadors SET databaixa=TS WHERE id_dvr=IDE;
  RSP:='OK';
  /*Insertem la informaci del procediment en el log*/
  INSERT INTO LOG_PROCEDIMENTS (ID_PRC,DATACRIDA,PENT1,PARAM_SORT)
  VALUES ('BXA_DVR',TS,TO_CHAR(IDE),RSP);
  /*Confirmem operacions*/  
  COMMIT;
  
  EXCEPTION
  WHEN OTHERS THEN
  ROLLBACK;
  /* Informem de l'error a la variable de sortida i guardem en el log de procediments*/
  RSP:=SUBSTR(sqlerrm,1,200);
  INSERT INTO LOG_PROCEDIMENTS (ID_PRC,DATACRIDA,PENT1,PARAM_SORT)
  VALUES ('BXA_DVR',TS,TO_CHAR(IDE),RSP);
  /*confirmem operacions*/
  COMMIT;
  
  
  
  NULL;
END BXA_DVR;

/
--------------------------------------------------------
--  DDL for Procedure BXA_USRS
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "USER1"."BXA_USRS" 
/*
Procediment del conjunt ABM d'usuaris. Aquest procediment dna de baixa un usuari del sistema.
s'insereix la data actual del sistema en el camp "databaixa" de la taula USUARIS, per l'usuari
concret.
*/
(
  IDUSR IN NUMBER  /*variable d'entrda. identifica l'usuari*/
, RSP OUT VARCHAR2  /*variable de sortida informativa.*/
) AS 
TS TIMESTAMP; /*variable local que guardar la data del sistema*/
BEGIN
TS:=SYSTIMESTAMP; /*inicialitzem la variable local*/
  /*Actualitzem la taula USUARIS inserint la data actual del sistema en el camp databaixa*/
  UPDATE USUARIS SET databaixa=TS WHERE id_usuari=IDUSR;
  RSP:='OK';
  /*Guardem la informaci del procediment en el log*/
  INSERT INTO LOG_PROCEDIMENTS (ID_PRC,DATACRIDA,PENT1,PARAM_SORT)
  VALUES ('BXA_USRS',TS,TO_CHAR(IDUSR),RSP);
  /*Confirmem operacions*/
  COMMIT;
  
  EXCEPTION
  WHEN OTHERS THEN
  ROLLBACK;
  /* Informem de l'error a la variable de sortida i guardem en el log de procediments*/
  RSP:=SUBSTR(sqlerrm,1,200);
  INSERT INTO LOG_PROCEDIMENTS (ID_PRC,DATACRIDA,PENT1,PARAM_SORT)
  VALUES ('BXA_USRS',TS,TO_CHAR(IDUSR),RSP);
  /*Confirmem operacions*/
  COMMIT;
  
END BXA_USRS;

/
--------------------------------------------------------
--  DDL for Procedure INS_APLICACIONS
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "USER1"."INS_APLICACIONS" 
/*
Procediment del conjunt ABM d'aplicacions. Aquest procediment insereix una aplicaci dins el sistema
A part d'inserir una aplicaci dins la taula APLICACIONS, tamb realitza altres operacions que sn:
a.Inserir en la taula APLIC_IDIOMA el nom de l'aplicaci en un idioma que per defecte s l'angls.
b.Inserir en la taula VERSIONS_APLICACIONS la versi de l'aplicaci que s'est inserint.
c.Inserim el sistema operatiu principal de l'aplicaci.
d.Inserim el principal desenvolupador, que ja tenim a la base de dades
e.Inserim una tarifa per a cada pas. Per defecte igual a tots els pasos.
*/
( 
  Vers VARCHAR2 /*versi de l'aplicaci que volem inserir */
, Enlla VARCHAR2 /*enlla de l'aplicaci */
, Url VARCHAR2  /* url de l'aplicaci*/
, Resol NUMBER /* resoluci mnima de l'aplicaci. Ha d'existir dins la taula RESOLUCIONS*/
, Act CHAR /* Activa o no activa. */
, Descrip VARCHAR2 /*Descripci en ANGLES de l'aplicaci que es vol inserir */
, OS NUMBER /* identificador del sistema operatiu. Ha d'existir dins la taula SISTEMES_OP */
, MIDAKB NUMBER /* mida en KB de l'aplicaci */
, IDDVR NUMBER /* identificador del desenvolupador principal.*/
, PREU NUMBER /* preu de l'aplicaci inicial en tots els pasos */
, RSP OUT VARCHAR2)  /*variable de sortida informativa */

AS

IDAPLIC NUMBER; /*variable local on es guardar l'identificador de l'aplicaci 
donat per la seqncia SEQ_APLICACIONS*/
TS TIMESTAMP; /*variable local per guardar la data del sistema per tot el procediment*/

BEGIN
TS:=SYSTIMESTAMP;
/*Guardem el valor id de la nova aplicaci per utilitzar-lo en diferents sentncies.*/
IDAPLIC:=SEQ_APLICACIONS.NEXTVAL;
/*Insertem a la taula aplicacions la nova aplicaci, segons els parmetres d'entrada.*/
INSERT INTO APLICACIONS (ID_APLICACIO,NOM,VERSIO,ENLLABIN,URL,ID_RESOLUCIO_MINIMA,ACTIVA,dataalta)
VALUES (IDAPLIC,Descrip,INS_APLICACIONS.Vers,INS_APLICACIONS.Enlla,INS_APLICACIONS.Url,INS_APLICACIONS.Resol,INS_APLICACIONS.Act,systimestamp);
/*Insertem dins la taula APLIC_IDIOMA la descripci de l'aplicaci que acabem de donar d'alta.*/
INSERT INTO APLIC_IDIOMA (ID_APLICACIO,ID_IDIOMA,DESCRIPCIO_APLIC) 
VALUES (IDAPLIC,'ENG',DESCRIP);
/*Insertem a la taula VERSIONS_APLICCIONS la nova aplicaci, segons els parmetres d'entrada.*/
INSERT INTO VERSIONS_APLICACIONS (ID_APLICACIO,VERSIO,ENLLABIN,URL,ID_RESOLUCIO_MINIMA,dataalta)
VALUES (IDAPLIC,INS_APLICACIONS.Vers,INS_APLICACIONS.Enlla,INS_APLICACIONS.Url,INS_APLICACIONS.Resol,TS);
/*Insertem el sistema operatiu principal de l'aplicaci. Si pot executar-se en ms sistemes, 
s'haur de fer en un altre procediment.*/
INSERT INTO aplic_so (ID_APLICACIO,ID_SISTEMA,MIDA_APLIC_KB) VALUES (IDAPLIC,OS,MIDAKB);
/*Insertem el principal desenvolupador, que ja tenim a la base de dades.*/
INSERT INTO aplic_dvr (ID_APLICACIO,ID_DVR) VALUES (IDAPLIC,IDDVR);
/*Insertem la mateixa tarifa per a tots els paisos*/
DECLARE
   CURSOR COUNTRIES is
      SELECT ID_PAIS FROM PAISOS;   
   var_idpais CHAR(2);
BEGIN
   OPEN COUNTRIES;
   LOOP
      FETCH COUNTRIES INTO var_idpais;
      EXIT WHEN COUNTRIES%NOTFOUND;  /* quan no s'ha trobat ms dades */                
      INSERT INTO TARIFES_APLIC_PAIS (ID_APLICACIO,PREU,ID_PAIS) VALUES (IDAPLIC, PREU, var_idpais);
   END LOOP;
   CLOSE COUNTRIES;
END;

/*Si arribem fins aqu sense cap problema, tot ha anat b.*/
RSP:='OK';
/*Insertem en la taula de log de procediments el nom del procediment, el moment actual, i les variables.*/
/*Vers VARCHAR2,Enlla VARCHAR2, Url VARCHAR2,Resol NUMBER,Act CHAR,Descrip VARCHAR2, OS NUMBER,MIDAKB NUMBER,IDDVR NUMBER,PREU NUMBER*/
INSERT INTO LOG_PROCEDIMENTS (ID_PRC,DATACRIDA,PENT1,PENT2,PENT3,PENT4,PENT5,PENT6,PENT7,PENT8,PENT9,PENT10,PARAM_SORT) 
VALUES ('INS_APLICACIONS',TS,Vers,Enlla,Url,Resol,Act,Descrip,OS,MIDAKB,IDDVR,PREU,RSP);
--Insertem en la taula TARIFES_APLIC_PAIS el preu inicial per a tots els pasos

/*confirmem operacions*/
COMMIT;


/*Tractem les excepcions.*/
EXCEPTION
WHEN OTHERS THEN
/*Si hi ha una excepci fem un rollback.*/
ROLLBACK;
/*Si hi ha qualsevol excepci l'anotem dins la variable RSP*/
RSP:=SUBSTR(sqlerrm,1,200);
/*Insertem en la taula de log de procediments el nom del procediment, 
el moment actual, i les variables.*/
INSERT INTO LOG_PROCEDIMENTS (ID_PRC,DATACRIDA,PENT1,PENT2,PENT3,PENT4,PENT5,PENT6,PENT7,PENT8,PENT9,PENT10,PARAM_SORT) 
VALUES ('INS_APLICACIONS',TS,Vers,Enlla,Url,Resol,Act,Descrip,OS,MIDAKB,IDDVR,PREU,RSP);
COMMIT;
END INS_APLICACIONS;

/
--------------------------------------------------------
--  DDL for Procedure INS_DVR
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "USER1"."INS_DVR" 
/*
Procediment del conjunt ABM de desenvolupadors. Aquest procediment insereix un 
desenvolupador a la taula DESENVOLUPADORS amb totes les seves dades.
*/
(
  NOM IN VARCHAR2  /* Nom del desenvolupador. Empresa o persona */
, REPRES IN VARCHAR2  /* Representant del desenvolupador */
, IDPAIS IN VARCHAR2  /* Identificador del pais 2 lletres */
, IDENTIF IN VARCHAR2  /* Identificador del desenvolupador */
, ADRE IN VARCHAR2  /* Adrea */ 
, CP IN VARCHAR2  /* Codi postal */
, POBLA IN VARCHAR2  /* Poblaci */
, PROV IN VARCHAR2 /* Provincia */
, TELF IN VARCHAR2 /* Telfon */
, MOBIL IN VARCHAR2   /* Mbil */ 
, FAX IN VARCHAR2  /* Fax */
, EMAIL IN VARCHAR2  /* e-mail */
, RSP OUT VARCHAR2 /* variable de sortida informativa */
) AS 

TS TIMESTAMP; /* Data del sistema en el moment de l'execuci del procediment*/
BEGIN
  
  /* Insertem les dades a la taula DESENVOLUPADORS */
  TS:=SYSTIMESTAMP;
  INSERT INTO desenvolupadors (ID_DVR,NOM,REPRESENTANT,ID_PAIS,NUM_IDENTIFICADOR,ADRE,CP,POBLACIO,PROVINCIA,TELF,MOBIL,FAX,EMAIL,DATAALTA) 
  VALUES (SEQ_DVR.NEXTVAL,NOM,REPRES,IDPAIS,IDENTIF,ADRE,CP,POBLA,PROV,TELF,MOBIL,FAX,EMAIL,TS);
  
  RSP:='OK'; /*si tot ha anat b*/
  
  /* Insertem a la taula LOG_PROCEDIMENTS */
  INSERT INTO LOG_PROCEDIMENTS (ID_PRC,DATACRIDA,PENT1,PENT2,PENT3,PENT4,PENT5,PENT6,PENT7,PENT8,PENT9,PENT10,PENT11,PENT12,PARAM_SORT) 
  VALUES ('INS_DVR',TS,NOM,REPRES,IDPAIS,IDENTIF,ADRE,CP,POBLA,PROV,TELF,MOBIL,FAX,EMAIL,RSP);
  /*Si tot ha anat b confirmem*/
  COMMIT;
  
  EXCEPTION
  WHEN OTHERS THEN
  ROLLBACK;
  /* Informem de l'error a la variable de sortida i guardem en el log de procediments*/
  RSP:=SUBSTR(sqlerrm,1,200);
  INSERT INTO LOG_PROCEDIMENTS (ID_PRC,DATACRIDA,PENT1,PENT2,PENT3,PENT4,PENT5,PENT6,PENT7,PENT8,PENT9,PENT10,PENT11,PENT12,PARAM_SORT) 
  VALUES ('INS_DVR',TS,NOM,REPRES,IDPAIS,IDENTIF,ADRE,CP,POBLA,PROV,TELF,MOBIL,FAX,EMAIL,RSP);
  /*Confirmem operacions*/
  COMMIT;
  
END INS_DVR;

/
--------------------------------------------------------
--  DDL for Procedure INS_USRS
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "USER1"."INS_USRS" 
/*
Procediment del conjunt ABM d'usuaris. Aquest procediment insereix un usuari en el sistema
amb totes les seves dades.

Nota: la inserci d'un nou usuari ha de comprova una de les taules d'estadstica per tal de 
preservar la integritat de les dades. Realitzem una comprovaci dins la taula 
estadstica ESTADISTICA_USR per comprovar que hi hagi dades inicialitzades. Si no hi sn s'hauran
d'inicialitzar.

*/
(
  NOM IN VARCHAR2  /* Nom de l'usuari*/
, COG1 IN VARCHAR2  /* primer cognom*/
, COG2 IN VARCHAR2  /* segon cognom*/
, ALIES IN VARCHAR2  /* alies dins la base de dades. No pot haver-hi dos iguals*/
, ADRE IN VARCHAR2  /* direcci de l'usuari*/
, CP IN VARCHAR2  /* codi postal*/
, EMAIL IN VARCHAR2  /* e-mail*/
, IDPAIS IN VARCHAR2  /* identificador del pas de l'usuari. Ha d'existir dins la taula PAISOS*/
, IDOPE IN VARCHAR2  /* identificador de l'operador. Ha d'existir dins la taula OPERADORS*/
, MOBIL IN VARCHAR2  /* nmero de telfon mbil de l'usuari. Segons la proposta del TFC noms hi ha un sol telfon per ususari i s intransferible*/
, CIFNIF IN VARCHAR2  /* CIF o NIF de l'usuari*/
, PWD IN VARCHAR2 /* contrasenya de l'usuari */
, IDIOMA IN CHAR /* identificador de l'idioma de l'usuari. Ha d'existir dins la taula IDIOMES*/
, RSP OUT VARCHAR2  /*variable de sortida informativa*/
) AS 
  TS TIMESTAMP; /*varible local per guardar la data actual del sistema*/
  EXISTEIX NUMBER; /*variable local operativa*/
  NUMUSRS NUMBER; /*variable local operativa*/
BEGIN
  TS:=SYSTIMESTAMP; /*inicialitzem la data local per utilitzarla en tot el procediment*/
  /*Inserim les dades de l'usuari a la taula USUARIS.*/
  INSERT INTO USUARIS (ID_USUARI,NOM,COGNOM1,COGNOM2,ALIES,DIRECCIO,CP,EMAIL,ID_PAIS,ID_OPERADOR,DATAALTA,MOBIL,CIF_NIF,PWD,IDIOMA)
  VALUES (SEQ_USUARIS.NEXTVAL,NOM,COG1,COG2,ALIES,ADRE,CP,EMAIL,IDPAIS,IDOPE,TS,MOBIL,CIFNIF,PWD,IDIOMA);
  /*guardem les dades del procediment*/
  RSP:='OK';
  INSERT INTO LOG_PROCEDIMENTS (ID_PRC,DATACRIDA,PENT1,PENT2,PENT3,PENT4,PENT5,PENT6,PENT7,PENT8,PENT9,PENT10,PENT11,PENT12,PENT13,PARAM_SORT) 
  VALUES ('INS_USRS',TS,NOM,COG1,COG2,ALIES,ADRE,CP,EMAIL,IDPAIS,IDOPE,MOBIL,CIFNIF,PWD,IDIOMA, RSP);
  
  /*
  Realitzem una comprovaci dins la taula estadstica ESTADISTICA_USR per comprovar que
  hi hagi dades inicialitzades (Si s el primer usuari que s'insereix de l'any no hi haur dades)
  */
  SELECT 1 INTO EXISTEIX FROM ESTADISTICA_USR WHERE PERIODE=EXTRACT(YEAR FROM TS);
  /*Si el select anterior retorna alguna dada aleshores actualitzem.*/
  UPDATE ESTADISTICA_USR SET N_usuaris=N_USUARIS+1 WHERE PERIODE=EXTRACT(YEAR FROM TS);
  /*confirmem operacions*/
  COMMIT;
  
  EXCEPTION
  /*Si no s'ha trobat cap valor en la taula ESTADISTICA_USR aleshores vol dir que haurem d'inserir un nou registre
  a aquesta taula, amb el valor del nombre d'usuaris igual a l'any anterior ms 1 (el que s'est inserint)*/
  WHEN NO_DATA_FOUND THEN
  /*busquem el nombre d'usuaris de la taula ESTADISTICA_USR del perode anterior i el guardem
  a la variable local NUMUSRS*/
  SELECT N_USUARIS INTO NUMUSRS FROM ESTADISTICA_USR WHERE PERIODE=EXTRACT(YEAR FROM TS)-1;
  /*Inserim el nombre d'usuaris trobat + 1 (l'actual) en un nou registre dins la taula
  ESTADISTICA_USR*/
  INSERT INTO ESTADISTICA_USR (PERIODE,N_DESCARREGUES,N_USUARIS,DATAMOD,DESCARREGUES_MITGES) VALUES
  (EXTRACT(YEAR FROM TS),0,NUMUSRS+1,TS,0); 
  /*Informem del resultat del procediment. Diferenciant de l'anterior resultat */
  RSP:='OK - nou registre';
  INSERT INTO LOG_PROCEDIMENTS (ID_PRC,DATACRIDA,PENT1,PENT2,PENT3,PENT4,PENT5,PENT6,PENT7,PENT8,PENT9,PENT10,PENT11,PENT12,PENT13,PARAM_SORT) 
  VALUES ('INS_USRS',TS,NOM,COG1,COG2,ALIES,ADRE,CP,EMAIL,IDPAIS,IDOPE,MOBIL,CIFNIF,PWD,IDIOMA, RSP);
 
  COMMIT;
  
  WHEN OTHERS THEN
  ROLLBACK;
   /* Informem de l'error a la variable de sortida i guardem en el log de procediments*/
  RSP:=SUBSTR(sqlerrm,1,200);
  INSERT INTO LOG_PROCEDIMENTS (ID_PRC,DATACRIDA,PENT1,PENT2,PENT3,PENT4,PENT5,PENT6,PENT7,PENT8,PENT9,PENT10,PENT11,PENT12,PENT13,PARAM_SORT) 
  VALUES ('INS_USRS',TS,NOM,COG1,COG2,ALIES,ADRE,CP,EMAIL,IDPAIS,IDOPE,MOBIL,CIFNIF,PWD,IDIOMA, RSP);
 
  COMMIT;
  
  NULL;
END INS_USRS;

/
--------------------------------------------------------
--  DDL for Procedure MOD_APLICACIONS
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "USER1"."MOD_APLICACIONS" 
/*
Procediment del conjunt ABM d'aplicacions. Aquest procediment modifica diferents camps de la taula
aplicacions. S'han d'informar tots els camps que s'indiquen en les variables d'entrada, en cas contrari
no funcionaria correctament. Actua sobre la taula APLICACIONS

NOTA: En el cas que hi hagi un canvi de versi tamb es veuria afectada la taula 
VERSIONS_APLICACIONS.

*/
(
 IDE NUMBER /**/
,Vers VARCHAR2 /**/
,Enlla VARCHAR2 /**/
,Url VARCHAR2 /**/
,Res NUMBER /**/
,Acti CHAR /**/
,RSP OUT VARCHAR2 /**/
)
AS
COMPROVA_RESOLUCIONS VARCHAR2(15); /**/
TS TIMESTAMP; /**/
BEGIN
TS:=SYSTIMESTAMP; /**/

/*Comprovem si la versi canvia...*/
SELECT VERSIO INTO COMPROVA_RESOLUCIONS FROM APLICACIONS WHERE ID_APLICACIO=IDE;
/*Si no canvia*/
IF COMPROVA_RESOLUCIONS=Vers THEN
/*Actualitzem la taula APLICACIONS*/
UPDATE APLICACIONS SET ENLLABIN=MOD_APLICACIONS.Enlla,URL=MOD_APLICACIONS.Url,ID_RESOLUCIO_MINIMA=MOD_APLICACIONS.Res,ACTIVA=MOD_APLICACIONS.Acti,DATAULTMOD=SYSTIMESTAMP 
 WHERE ID_APLICACIO=MOD_APLICACIONS.IDE;
 RSP:='OK';
INSERT INTO LOG_PROCEDIMENTS (ID_PRC,DATACRIDA,PENT1,PENT2,PENT3,PENT4,PENT5,PENT6,PARAM_SORT) 
VALUES ('MOD_APLICACIONS',TS,TO_CHAR(IDE),Vers,Enlla,Url,Res,Acti,RSP);
/*Si canvia. Insertem a la taula VERSIONS_APLICACIONS la nova versi, segons els parmetres d'entrada*/
ELSE
INSERT INTO VERSIONS_APLICACIONS (ID_APLICACIO,VERSIO,ENLLABIN,URL,ID_RESOLUCIO_MINIMA,dataalta)
VALUES (IDE,Vers,Enlla,Url,Res,TS);
/*I desprs actualitzem la taula APLICACIONS.*/
UPDATE APLICACIONS SET VERSIO=MOD_APLICACIONS.Vers,ENLLABIN=MOD_APLICACIONS.Enlla,URL=MOD_APLICACIONS.Url,ID_RESOLUCIO_MINIMA=MOD_APLICACIONS.Res,ACTIVA=MOD_APLICACIONS.Acti,DATAULTMOD=SYSTIMESTAMP 
 WHERE ID_APLICACIO=MOD_APLICACIONS.IDE;

END IF;

/*Guardem les dades del procediment*/
RSP:='OK';
INSERT INTO LOG_PROCEDIMENTS (ID_PRC,DATACRIDA,PENT1,PENT2,PENT3,PENT4,PENT5,PENT6,PARAM_SORT) 
VALUES ('MOD_APLICACIONS',TS,TO_CHAR(IDE),Vers,Enlla,Url,Res,Acti,RSP);

/*confirmem operacions*/
COMMIT;

EXCEPTION
WHEN OTHERS THEN
/*retrocedim possibles canvis*/
ROLLBACK;
/* Informem de l'error a la variable de sortida i guardem en el log de procediments*/
RSP:=SUBSTR(sqlerrm,1,200);
INSERT INTO LOG_PROCEDIMENTS (ID_PRC,DATACRIDA,PENT1,PENT2,PENT3,PENT4,PENT5,PENT6,PARAM_SORT) 
VALUES ('MOD_APLICACIONS',TS,TO_CHAR(IDE),Vers,Enlla,Url,Res,Acti,RSP);
/*confirmem operacions*/
COMMIT;

END MOD_APLICACIONS;

/
--------------------------------------------------------
--  DDL for Procedure MOD_DVR
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "USER1"."MOD_DVR" 
/*
Procediment del conjunt ABM de desenvolupadors. Aquest procediment modifica les dades del 
desenvolupador escollit. S'han d'informar de totes les variables, encara que no canvin.
En cas contrari no funcionaria.
*/
(
  IDE IN NUMBER /* identificador del desenvolupador */
, NOM IN VARCHAR2  /* nom */
, REPRES IN VARCHAR2  /* representant */
, IDPAIS IN VARCHAR2  /* identificador del pais*/
, IDENTIF IN VARCHAR2  /* CIF, NIF, etc del desenvolupador */
, ADRE IN VARCHAR2  /* direcci */ 
, CP IN VARCHAR2  /* codi postal */
, POBLA IN VARCHAR2  /* poblaci */
, PROV IN VARCHAR2 /* provincia */
, TELF IN VARCHAR2 /* telfon */
, MOBIL IN VARCHAR2   /* nmero mbil*/ 
, FAX IN VARCHAR2  /* nmero de fax */
, EMAIL IN VARCHAR2  /* e-mail*/
, RSP OUT VARCHAR2 /* variable de sortida informativa */
) AS  

TS TIMESTAMP; /*variable local per la data del sistema*/
BEGIN
  TS:=SYSTIMESTAMP; /*Inicialitzem la variable local amb la data del sistema*/
  UPDATE desenvolupadors SET
    NOM=MOD_DVR.NOM,
    REPRESENTANT=REPRES,
    ID_PAIS=IDPAIS,
    num_identificador=IDENTIF,
    ADRE=mod_dvr.ADRE,
    CP=MOD_DVR.CP,
    POBLACIO=POBLA,
    PROVINCIA=PROV,
    TELF=MOD_DVR.TELF,
    MOBIL=MOD_DVR.MOBIL,
    FAX=MOD_DVR.FAX,
    EMAIL=MOD_DVR.EMAIL,
    DATAULTMOD=TS
  WHERE ID_DVR=IDE;
  
  /*Informem en el log de procediments*/
  RSP:='OK';
  INSERT INTO LOG_PROCEDIMENTS (ID_PRC,DATACRIDA,PENT1,PENT2,PENT3,PENT4,PENT5,PENT6,PENT7,PENT8,PENT9,PENT10,PENT11,PENT12,PENT13,PARAM_SORT) 
  VALUES ('MOD_DVR',TS,TO_CHAR(IDE),NOM,REPRES,IDPAIS,IDENTIF,ADRE,CP,POBLA,PROV,TELF,MOBIL,FAX,EMAIL,RSP);
  /*Confirmem operacions*/
  COMMIT;
  
  EXCEPTION
  WHEN OTHERS THEN
  /*retrocedim possibles canvis*/
  ROLLBACK;
  /* Informem de l'error a la variable de sortida i guardem en el log de procediments*/
  RSP:=SUBSTR(sqlerrm,1,200);
  INSERT INTO LOG_PROCEDIMENTS (ID_PRC,DATACRIDA,PENT1,PENT2,PENT3,PENT4,PENT5,PENT6,PENT7,PENT8,PENT9,PENT10,PENT11,PENT12,PENT13,PARAM_SORT) 
  VALUES ('INS_DVR',TS,TO_CHAR(IDE),NOM,REPRES,IDPAIS,IDENTIF,ADRE,CP,POBLA,PROV,TELF,MOBIL,FAX,EMAIL,RSP);
  
  /*confirmem operacions*/
  COMMIT;
  
  
  NULL;
END MOD_DVR;

/
--------------------------------------------------------
--  DDL for Procedure MOD_USRS
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "USER1"."MOD_USRS" 
/*
Procediment ABM d'usuaris. Aquest procediment modifica la informaci dels ususaris.
*/
(
  IDUSR IN VARCHAR2  /**/
,NOMUSR IN VARCHAR2  /**/
, COG1 IN VARCHAR2  /**/
, COG2 IN VARCHAR2  /**/
, ALIESUSR IN VARCHAR2  /**/
, ADRE IN VARCHAR2  /**/
, CPUSR IN VARCHAR2  /**/
, EMAILUSR IN VARCHAR2  /**/
, IDPAIS IN VARCHAR2  /**/
, IDOPE IN VARCHAR2  /**/
, MOBILUSR IN VARCHAR2  /**/
, CIFNIF IN VARCHAR2  /**/
, PWDUSR IN VARCHAR2
, IDI IN CHAR /**/
, RSP OUT VARCHAR2  /**/ 
) AS 

TS TIMESTAMP;
BEGIN
TS:=SYSTIMESTAMP;
  UPDATE USUARIS SET NOM=NOMUSR,COGNOM1=COG1,
  COGNOM2=COG2,ALIES=ALIESUSR,DIRECCIO=ADRE,CP=CPUSR,
  EMAIL=EMAILUSR,ID_PAIS=IDPAIS,ID_OPERADOR=IDOPE,DATAULTMOD=TS,
  MOBIL=MOBILUSR,CIF_NIF=CIFNIF,PWD=PWDUSR,IDIOMA=IDI 
  WHERE ID_USUARI=IDUSR;
  RSP:='OK';
  
   INSERT INTO LOG_PROCEDIMENTS (ID_PRC,DATACRIDA,PENT1,PENT2,PENT3,PENT4,PENT5,PENT6,PENT7,PENT8,PENT9,PENT10,PENT11,PENT12,PENT13,PENT14,PARAM_SORT) 
  VALUES ('MOD_USRS',TS,IDUSR,NOMUSR,COG1,COG2,ALIESUSR,ADRE,CPUSR,EMAILUSR,IDPAIS,IDOPE,MOBILUSR,CIFNIF,PWDUSR,IDI, RSP);
  COMMIT;
  
  EXCEPTION
  WHEN OTHERS THEN
  ROLLBACK;
  /* Informem de l'error a la variable de sortida i guardem en el log de procediments*/
   RSP:=SUBSTR(sqlerrm,1,200);
   INSERT INTO LOG_PROCEDIMENTS (ID_PRC,DATACRIDA,PENT1,PENT2,PENT3,PENT4,PENT5,PENT6,PENT7,PENT8,PENT9,PENT10,PENT11,PENT12,PENT13,PENT14,PARAM_SORT) 
  VALUES ('MOD_USRS',TS,IDUSR,NOMUSR,COG1,COG2,ALIESUSR,ADRE,CPUSR,EMAILUSR,IDPAIS,IDOPE,MOBILUSR,CIFNIF,PWDUSR,IDI, RSP);
  COMMIT;
  
  

  NULL;
END MOD_USRS;

/
--------------------------------------------------------
--  DDL for Procedure PRC_DESCARREGUES
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "USER1"."PRC_DESCARREGUES" 
/*
Procediment de descrregues. Aquest procediment ens ha de servir per incorporar totes les
descrregues de les aplicacions a la base de dades. Es tracta d'un procediment que fa el segent:
a.Actualitza la taula USUARIS sumant 1 descrrega al camp "numdesc" de l'usuari en concret
b.Actualitza la taula APLICACIONS sumant 1 descrrega al camp "numdesc" de l'aplicaci que es descarrega
c.Insereix un nou registre a la taula DESCARREGUES.

*/
(
  IDUSR IN NUMBER /*variable d'entrada que representa l'identificador de l'usuari*/ 
, IDAPLIC IN NUMBER /*variabl d'entrada. identificador de l'aplicaci*/  
, VERS IN VARCHAR2  /*variable d'entrada. versi de l'aplicaci*/
, PREU IN NUMBER  /*variable d'entrada. preu de l'aplicaci*/
, IDFP IN VARCHAR2  /*variable d'entrada. identificador forma de pagament*/
, IMEI IN VARCHAR2  /*variable d'entrada. imei del dispositiu de l'usuari.*/
, IDOPERA IN VARCHAR2  /*vairable d'entrada. operador de l'usuari*/
, IDDIVISA IN VARCHAR2 /*variable d'entrada. divisa amb la qual s'opera*/
, RSP OUT VARCHAR2/**/
) AS 
TS TIMESTAMP; /*variable local per la data del sistema*/
PAIS CHAR(2); /*variable local per guardar un identificador de pais.*/
EXISTEIX NUMBER;

BEGIN
TS:=SYSTIMESTAMP; /*Inicialitzem la variable data del sistema local*/
  /*Necessitem saber de quin pais s l'usuari*/
  SELECT ID_PAIS INTO PAIS FROM USUARIS WHERE ID_USUARI=IDUSR;
   
   /*Actualitzem USUARIS abans que es desencadenin els triggers de la descrrega*/
  UPDATE USUARIS SET NUMDESC=NUMDESC+1,ULTDESC=TS,IMPORTTOTAL=IMPORTTOTAL+PREU WHERE ID_USUARI=IDUSR;
   /*Actualitzem APLICACIONS desprs que es desencadenin els triggers de la descrrega*/
  UPDATE APLICACIONS SET NUMDESC=NUMDESC+1,ULTDESC=TS,IMPORTTOTAL=IMPORTTOTAL+PREU WHERE  ID_APLICACIO=IDAPLIC;
  /*Finalment insertem en la taula descrrega i aix ocasiona que el disparador TRG_DESCARREGUES comenci a funcionar*/
 
  /*Inserim el nou registre a la taula DESCARREGUES*/
  INSERT INTO DESCARREGUES (ID_DESCARREGA,ID_USUARI,ID_APLICACIO,VERSIO,PREUPAGAT,ID_FP,IMEI,ID_OPERADOR,ID_PAIS,ID_DIVISA)
  VALUES (SEQ_DESCARREGA.NEXTVAL,IDUSR,IDAPLIC,VERS,PREU,IDFP,IMEI,IDOPERA,PAIS,IDDIVISA);
 
  /*Tot s'ha executat correctament*/
  RSP:='OK';
  /*Guardem les dades rellevants en el log de procediments*/
  INSERT INTO log_procediments (ID_PRC,DATACRIDA,PENT1,PENT2,PENT3,PENT4,PENT5,PENT6,PENT7,PENT8,PARAM_SORT) 
  VALUES ('PRC_DESCARREGUES',TS,IDUSR,IDAPLIC,VERS,PREU,IDFP,IMEI,IDOPERA,IDDIVISA,RSP);
  /*Si no hi ha hagut cap error es confirma tot*/
  COMMIT;
  
  /*En cas de qualsevol excepci no controlada*/
  EXCEPTION
   WHEN OTHERS THEN
  /*Davant qualsevol error en l'execuci es fa un ROLLBACK*/
  ROLLBACK;
  /* Informem de l'error a la variable de sortida i guardem en el log de procediments*/
  RSP:=SUBSTR(sqlerrm,1,200);
  INSERT INTO log_procediments (ID_PRC,DATACRIDA,PENT1,PENT2,PENT3,PENT4,PENT5,PENT6,PENT7,PENT8,PARAM_SORT) 
  VALUES ('PRC_DESCARREGUES',TS,IDUSR,IDAPLIC,VERS,PREU,IDFP,IMEI,IDOPERA,IDDIVISA,RSP);
  /*confirmem les operacions desde EXCEPTION*/
  COMMIT;
END PRC_DESCARREGUES;

/
--------------------------------------------------------
--  DDL for Procedure PRC_ESTAD_1
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "USER1"."PRC_ESTAD_1" 
/*
Procediment que actualitza el nmero total de descrregues de la plataforma. es desencadena mitjanant el disparador
TRG_DESCARREGUES. taula ESTADISTICA_GENERAL 
*/
(
  RSP OUT VARCHAR2  /* variable de sortida informativa */
  
) AS
TS TIMESTAMP; /*variable que guarda la data del sistema*/
BEGIN
  TS:=SYSTIMESTAMP;
  /*Premisa: La taula ESTADISITCA_GENERAL ha d'estar inicialitzada.*/
  UPDATE ESTADISTICA_GENERAL SET descar_totals_acum=descar_totals_acum+1;
  /*Informem del bon resultat a la variable de sortida i guardem el registre al log de procediments.*/
  RSP:='OK';
  INSERT INTO log_procediments (ID_PRC,DATACRIDA,PARAM_SORT) VALUES ('PRC_ESTAD_1',TS,RSP);
    
  /*Si succeeix algun tipus d'excepci no controlada*/
  EXCEPTION
  WHEN OTHERS THEN
  /* Informem de l'error a la variable de sortida i guardem en el log de procediments*/
   RSP:=SUBSTR(sqlerrm,1,200);
  INSERT INTO log_procediments (ID_PRC,DATACRIDA,PARAM_SORT) VALUES ('PRC_ESTAD_1',TS,RSP);
END PRC_ESTAD_1;

/
--------------------------------------------------------
--  DDL for Procedure PRC_ESTAD_2
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "USER1"."PRC_ESTAD_2" 
/*
Procediment que actualitza el preu total pagat en descrregues de la plataforma. es desencadena mitjanant el disparador
TRG_DESCARREGUES. taula ESTADISTICA_GENERAL
*/
(
  PREUPAGAT IN NUMBER, /*variable d'entrada que informa sobre el preu pagat per l'usuari*/
  RSP OUT VARCHAR2  /*variable de sortida informativa*/
  
) AS
TS TIMESTAMP;
BEGIN
  TS:=SYSTIMESTAMP;
  /*La taula ESTADISTICA_GENERAL ha d'estar inicialitzada.
  S'actualitza el camp IMPORT_TOTAL sumant el preu pagat en la descrrega en curs.*/
  UPDATE ESTADISTICA_GENERAL SET IMPORT_TOTAL=IMPORT_TOTAL+PREUPAGAT;
  /*Tot ha funcionat correctament*/
  RSP:='OK';
  INSERT INTO log_procediments (ID_PRC,DATACRIDA,PENT1,PARAM_SORT) VALUES ('PRC_ESTAD_2',TS,PREUPAGAT,RSP);
  /*Qualsevol excepci no controlada far abortar el procediment i ho registrarem al log*/  
  EXCEPTION
  WHEN OTHERS THEN
  /* Informem de l'error a la variable de sortida i guardem en el log de procediments*/
  RSP:=SUBSTR(sqlerrm,1,200);
  INSERT INTO log_procediments (ID_PRC,DATACRIDA,PENT1,PARAM_SORT) VALUES ('PRC_ESTAD_2',TS,PREUPAGAT,RSP);
END PRC_ESTAD_2;

/
--------------------------------------------------------
--  DDL for Procedure PRC_ESTAD_3
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "USER1"."PRC_ESTAD_3" (
/*Procediment que permet actualitzar la informaci, just desprs de fer una descrrega, per tal de poder calcular el nmero mig d'aplicacions
per usuari en un any concret. Punt 3 del mdul estadstic. Taula ESTADISTICA_USR*/

  DATADESC IN TIMESTAMP, /*variable d'entrada que informa de la data de la descrrega*/
  RSP OUT VARCHAR2 /*variable de sortida informativa*/
)  AS
TS TIMESTAMP; /*variable local amb la data del sistema*/
NUMUSRS NUMBER; /*variable local que guardar els usuaris del perode anterior en cas de canvi de perode*/
EXISTEIX NUMBER; /*variable local procedimental*/
BEGIN
  /*data actual del sistema. Ens la guardem per utilitzar-la en tot el procediment*/
  TS:=SYSTIMESTAMP;
  /*Comprovem que existeixi un registre amb l'any actual de la descrrega. Si el select
  dna algun resultat, seguim amb les actualitzacions. En cas contrari passem a l'excepci
  controlada NO_DATA_FOUND*/  
  SELECT 1 INTO EXISTEIX FROM ESTADISTICA_USR WHERE  PERIODE=EXTRACT(YEAR FROM DATADESC);
   /*Si no ha donat error NO_DATA_FOUND podem actualitzar la taula ESTADISTICA_USR incrementant la descrrega en una unitat
   i recalculant les descrregues mitges */   
  UPDATE ESTADISTICA_USR SET N_DESCARREGUES=N_DESCARREGUES+1 WHERE PERIODE=EXTRACT(YEAR FROM DATADESC);
  UPDATE ESTADISTICA_USR SET DESCARREGUES_MITGES=N_DESCARREGUES/N_USUARIS,DATAMOD=TS WHERE PERIODE=EXTRACT(YEAR FROM DATADESC);
    /*Si tot ha funcionat correctament a podem procedir a guardar la informaci del procediment amb resultat OK*/
    RSP:='OK';
     /*Reistrem en el log dels procediments*/
    INSERT INTO log_procediments (ID_PRC,DATACRIDA,PENT1,PARAM_SORT) VALUES ('PRC_ESTAD_3',TS,DATADESC,RSP);
  
  
  EXCEPTION
    /*NO_DATA_FOUND quan el SELECT no dna resultats. Voldr dir que no s'ha registrat cap nou usuari durant aquest perode, ja que
  per defecte el registre nou del perode es crea en el procediment INS_USRS*/
  WHEN NO_DATA_FOUND THEN 
    /*Insertarem un nou registre per perode actual*/
    INSERT INTO ESTADISTICA_USR (PERIODE,N_DESCARREGUES,N_USUARIS,DATAMOD,DESCARREGUES_MITGES) 
    VALUES (EXTRACT(YEAR FROM DATADESC),0,0,TS,0);
    /*Seleccionem els usuaris del perode anterior, que han de ser exactament els usuaris actuals*/
    SELECT N_USUARIS INTO NUMUSRS FROM ESTADISTICA_USR WHERE  PERIODE=EXTRACT(YEAR FROM DATADESC)-1;
    /*Actualitzem el registre insertat amb les dades de 1 nova descrrega i els usuaris trobats del perode anterior */
    UPDATE ESTADISTICA_USR SET N_DESCARREGUES=1,N_USUARIS=NUMUSRS WHERE PERIODE=EXTRACT(YEAR FROM DATADESC);
    /*Actualitzem la mitjana de descrregues.*/
    UPDATE ESTADISTICA_USR SET DESCARREGUES_MITGES=N_DESCARREGUES/N_USUARIS WHERE PERIODE=EXTRACT(YEAR FROM DATADESC);
  /*Tot ha anat correctament dins de l'excepci controlada i ho notifiquem en la variable de sortida*/
  RSP:='OK - Nou registre';
  /*Registrem en el log dels procediments*/
  INSERT INTO log_procediments (ID_PRC,DATACRIDA,PENT1,PARAM_SORT) VALUES ('PRC_ESTAD_3',TS,DATADESC,RSP);
    
  WHEN OTHERS THEN
  /* Informem de l'error a la variable de sortida i guardem en el log de procediments*/
  RSP:=SUBSTR(sqlerrm,1,200);
  INSERT INTO log_procediments (ID_PRC,DATACRIDA,PARAM_SORT) VALUES ('PRC_ESTAD_3',TS,RSP);
  RAISE;
END PRC_ESTAD_3;

/
--------------------------------------------------------
--  DDL for Procedure PRC_ESTAD_4
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "USER1"."PRC_ESTAD_4" 
/*Procediment del punt 4 del mdul estadstic. Permet mantenir en temps real l'estadstica demanada en la qual donat un any concret
volem saber el desenvolupador amb ms descrregues, sumant totes les descrregues de totes les seves aplicacions. Tenint en compte que 
quan es fa una descrrega ens podem trobar que l'aplicaci descarregada tingui ms d'un desenvolupador. 
taula ESTADISTICA_DVR*/
(
  ANYY IN NUMBER  /*variable d'entrada amb l'any de la descrrega*/
, IDAPL IN NUMBER  /*variable d'entrada identificador de l'aplicaci*/
, RSP OUT VARCHAR2  /*variable de sortida informativa*/
) AS 
/*Cursor que ens servir per saber tots els desenvolupadors d'una aplicaci en el moment actual*/
CURSOR DVRS IS SELECT ID_DVR FROM APLIC_DVR WHERE ID_APLICACIO=IDAPL;
IDDVR DVRS%ROWTYPE; /*variable per guardar les files del cursor anterior i operar amb elles.*/
/*Cursor que ens servir per saber tots els desenvolupadors d'una aplicaci que ja tenen registre 
a la taula d'estadstica en el perode de la descrrega*/
CURSOR DVRS2 IS SELECT ID_DVR FROM APLIC_DVR WHERE ID_APLICACIO=IDAPL AND ID_DVR 
NOT IN (SELECT ID_DVR FROM ESTADISTICA_DVR WHERE  PERIODE=ANYY);
IDDVR2 DVRS2%ROWTYPE;  /*variable per guardar les files del cursor anterior i operar amb elles.*/
TS TIMESTAMP; /*variable amb la data del sistema*/
EXISTEIX NUMBER; /*variable per operar en el procediment*/

BEGIN
 TS:=SYSTIMESTAMP; /*donem valor a la variable i ens servir per tot el procediment*/
 /*Amb l'obertura d'aquest cursor pretenem incorporar a la taula d'estadstica els desenvolupadors que s'haurien afegit
 a la taula APLIC_DVR com a nous desenvolupadors de l'aplicaci que s'est descarregant*/
 OPEN DVRS2;
  LOOP
    FETCH DVRS2 INTO IDDVR2;
    EXIT WHEN DVRS2%NOTFOUND;
      INSERT INTO ESTADISTICA_DVR (PERIODE,ID_DVR,N_DESCARREGUES) VALUES (ANYY,IDDVR2.ID_DVR,0);
  END LOOP ;
  CLOSE DVRS2;
 
 /*Procedim a actualitzar la taula ESTADISTICA_DVR. S'actualitzen tots els desenvolupadors assignats a l'aplicaci
 que s'est descarregant*/
 OPEN DVRS;
LOOP
 FETCH DVRS INTO IDDVR;
 EXIT WHEN DVRS%NOTFOUND;
 UPDATE ESTADISTICA_DVR SET N_DESCARREGUES=N_DESCARREGUES+1 WHERE PERIODE=ANYY AND ID_DVR=IDDVR.ID_DVR;
END LOOP ;
 CLOSE DVRS;
 /*Tot ha funcionat correctament*/
 RSP:='OK';
 INSERT INTO log_procediments (ID_PRC,DATACRIDA,PENT1,PENT2,PARAM_SORT) VALUES ('PRC_ESTAD_4',TS,TO_CHAR(ANYY),TO_CHAR(IDAPL),RSP);

/*Si succeeix una excepci no controlada*/
EXCEPTION
 
  WHEN OTHERS THEN
  /* Informem de l'error a la variable de sortida i guardem en el log de procediments*/
  RSP:=SUBSTR(sqlerrm,1,200);
  INSERT INTO log_procediments (ID_PRC,DATACRIDA,PENT1,PENT2,PARAM_SORT) VALUES ('PRC_ESTAD_4',TS,TO_CHAR(ANYY),TO_CHAR(IDAPL),RSP);

END PRC_ESTAD_4;

/
--------------------------------------------------------
--  DDL for Procedure PRC_ESTAD_5
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "USER1"."PRC_ESTAD_5" 
/*Procediment per tal que donat un any concret, poguem informar de l'aplicaci que ms ha recaudat en temps real. 
Punt 5 del mdul estadstic. taula ESTADISTICA_APL */
(
  ANYY IN NUMBER  /* identificador del perode de la descrrega */
, IDAPL IN NUMBER /* Identificador de l'aplicaci que es descarrega */
, PREU IN NUMBER /* Preu pagat per l'usuari en la descrrega */ 
, RSP OUT VARCHAR2  /*variable de sortida que guarda el resultat de l'execuci del procediment*/
) AS 

TS TIMESTAMP;  /*variable local que guarda la data del sistema*/
EXISTEIX NUMBER; /*variable local operativa*/
BEGIN
TS:=SYSTIMESTAMP;
 /*Es comprova si existeix un registre a la taula ESTADISTICA_APL de l'aplicaci que s'est 
 descarregant en el perode de la descrrega actual*/
 SELECT 1 INTO EXISTEIX FROM ESTADISTICA_APL WHERE PERIODE=ANYY AND ID_APLICACIO=IDAPL;
 
 /*Si existeix s'actualitza el valor del camp EUROS_RECAUDATS sumant el PREU pagat per l'usuari
 en la descrrega. En cas que no existeixi passem a l'excepci controlada NO_DATA_FOUND*/  
 UPDATE ESTADISTICA_APL SET EUROS_RECAUDATS=EUROS_RECAUDATS+PREU WHERE PERIODE=ANYY AND ID_APLICACIO=IDAPL;
 /*Tot ha anat b. S'actualitza el registre del log*/
 RSP:='OK';
 INSERT INTO log_procediments (ID_PRC,DATACRIDA,PENT1,PENT2,PENT3,PARAM_SORT) VALUES ('PRC_ESTAD_5',TS,TO_CHAR(ANYY),TO_CHAR(IDAPL),PREU,RSP);
 
  EXCEPTION
  WHEN NO_DATA_FOUND THEN
  /* Si no trobem el registre en el periode de la descrrega per l'aplicaci que s'est descarregant
  inserim un nou registre a la taula ESTADISTICA_APL */
  INSERT INTO ESTADISTICA_APL (PERIODE,ID_APLICACIO,EUROS_RECAUDATS) VALUES (ANYY,IDAPL,PREU);
  /*Tot ha anat b dins l'excepci controlada. Ho notifiquem en la variable de sortida*/
  RSP:='OK - Nou registre ESTADISTICA_APL';
  INSERT INTO log_procediments (ID_PRC,DATACRIDA,PENT1,PENT2,PENT3,PARAM_SORT) VALUES ('PRC_ESTAD_5',TS,TO_CHAR(ANYY),TO_CHAR(IDAPL),PREU,RSP);
 
  /*Si ha succet qualsevol altra excepci es notifica l'error en la variable RSP i es guarda en el
  log de procediments. */
  WHEN OTHERS THEN
  /* Informem de l'error a la variable de sortida i guardem en el log de procediments*/
  RSP:=SUBSTR(sqlerrm,1,200);
  INSERT INTO log_procediments (ID_PRC,DATACRIDA,PENT1,PENT2,PENT3,PARAM_SORT) VALUES ('PRC_ESTAD_5',TS,TO_CHAR(ANYY),TO_CHAR(IDAPL),PREU,RSP);

END PRC_ESTAD_5;

/
--------------------------------------------------------
--  DDL for Procedure PRC_ESTAD_6
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "USER1"."PRC_ESTAD_6" 
/*Prodeciment del punt 6 del mdul estadstic. Actualitza en temps real el nombre d'usuaris diferents que han fet una descrrega
donat un any concret i un pas. taula ESTADISTICA_ANY_PAIS*/
(
 ANYY IN NUMBER /**/
, IDUSR IN NUMBER /**/
, PAIS IN CHAR /**/
, RSP OUT VARCHAR2 /**/  )
AS 

TS TIMESTAMP; /*variable local que guardar la data del sistema a l'executar el procediment*/
EXISTEIX NUMBER; /* variable local operativa*/
NUMDESMAJORZERO number; /* variable local que contindr el nombre de descrregues de l'usuari*/

BEGIN
/**/
TS:=SYSTIMESTAMP;
/**/
SELECT 1 INTO EXISTEIX FROM estadistica_any_pais WHERE PERIODE=ANYY AND id_pais=PAIS;
/*Inicialitzem la variable a 0*/
NUMDESMAJORZERO:=0;
/*Premisa: l'usuari ha d'existir forosament a la base de dades per arribar fins aqu. Busquem
en la taula d'usuaris si aquest usuari ha realitzat mai cap descrrega en el perode actual*/
SELECT NUMDESC INTO NUMDESMAJORZERO  FROM USUARIS WHERE id_USUARI=IDUSR and EXTRACT(YEAR FROM (ultdesc))=EXTRACT(YEAR FROM(TS));
/**/
IF NUMDESMAJORZERO<=1 THEN
/**/
UPDATE estadistica_any_pais SET n_dif_usuaris=n_dif_usuaris+1 WHERE PERIODE=ANYY AND id_pais=PAIS;
RSP:='OK - n_dif_usuaris modificat UPDATE';
ELSE
RSP:='OK - n_dif_usuaris no modificat ';
END IF;
INSERT INTO log_procediments (ID_PRC,DATACRIDA,PENT1,PENT2,PENT3,PARAM_SORT) VALUES ('PRC_ESTAD_6',TS,IDUSR,ANYY,PAIS,RSP);
/**/
  EXCEPTION
WHEN NO_DATA_FOUND THEN
/*Insertem un registre nou degut a que no se n'ha trobat cap.*/
INSERT INTO estadistica_any_pais (id_pais,import_total,n_dif_aplicacions,n_dif_usuaris,periode) 
VALUES (PAIS,0,0,1,ANYY);
RSP:='OK - n_dif_usuaris modificat INSERT';

/**/
INSERT INTO log_procediments (ID_PRC,DATACRIDA,PENT1,PENT2,PENT3,PARAM_SORT) VALUES ('PRC_ESTAD_6',TS,IDUSR,ANYY,PAIS,RSP);

WHEN OTHERS THEN
 /* Informem de l'error a la variable de sortida i guardem en el log de procediments*/
  RSP:=SUBSTR(sqlerrm,1,200);
  INSERT INTO log_procediments (ID_PRC,DATACRIDA,PENT1,PENT2,PENT3,PARAM_SORT) VALUES ('PRC_ESTAD_6',TS,IDUSR,ANYY,PAIS,RSP);

END PRC_ESTAD_6;

/
--------------------------------------------------------
--  DDL for Procedure PRC_ESTAD_7
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "USER1"."PRC_ESTAD_7" 
/*
Procediment del punt 7 del mdul estadstic. Actualitza en temps real per tal que donat un any concret
i un pas, sapiguem els ingressos totals que han generat els usuaris registrats en aquell pas en descrreuges.
taula que es modifica ESTADISTICA_ANY_PAIS
*/
(
  ANYY IN NUMBER /**/
, PAIS IN VARCHAR2 /**/
, PREU IN NUMBER /**/
, RSP OUT VARCHAR2  /**/ 
) AS 
TS TIMESTAMP; /**/
EXISTEIX NUMBER; /**/
BEGIN 
TS:=SYSTIMESTAMP; /**/

/*Simplement actualitzem la taula ESTADISTICA_ANY_PAIS amb el preu que ha pagat l'usuari al pas que li correspongui*/
UPDATE estadistica_any_pais SET import_total=import_total+PREU WHERE PERIODE=ANYY AND id_pais=PAIS;
RSP:='OK';
INSERT INTO log_procediments (ID_PRC,DATACRIDA,PENT1,PENT2,PENT3,PARAM_SORT) VALUES ('PRC_ESTAD_7',TS,ANYY,PAIS,PREU,RSP);

EXCEPTION
/*Si succeeix una excepci no controlada*/
  WHEN OTHERS THEN
  /* Informem de l'error a la variable de sortida i guardem en el log de procediments*/
  RSP:=SUBSTR(sqlerrm,1,200);
  INSERT INTO log_procediments (ID_PRC,DATACRIDA,PENT1,PENT2,PENT3,PARAM_SORT) VALUES ('PRC_ESTAD_7',TS,ANYY,PAIS,PREU,RSP);
  NULL;
END PRC_ESTAD_7;

/
--------------------------------------------------------
--  DDL for Procedure PRC_ESTAD_8
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "USER1"."PRC_ESTAD_8" 
/*
Procediment que actualitza la taula ESTADISTICA_ANY_PAIS d'acord amb el que es demana en el modul estadstic
apartat 8. "Donat un any concret i un pais, el nmero d'aplicacions diferents descarregades com a mnim una vegada.
*/
(
 ANYY IN NUMBER /*variable on s'informa de l'any*/
, IDAPLIC IN NUMBER /*variable identificador de l'aplicaci*/
, PAIS IN CHAR /*variable identificador del pais*/
, RSP OUT VARCHAR2 /*variable de sortida*/  )
AS 

TS TIMESTAMP; /* Variable local que guarda la data del sistema*/
EXISTEIX NUMBER; /* variable local operativa */

BEGIN
TS:=SYSTIMESTAMP; /*guardem la data del sistema per utilitzar-la en diferents apartats del codi*/

/* Realitzem una comprovaci per tal de saber si existeix una descrrega a la taula APLIC_ANY_PAIS 
que correspongui amb l'aplicaci que s'est descarregant, el perode actual i el pas de l'usuari.*/

SELECT 1 INTO  EXISTEIX FROM APLIC_ANY_PAIS WHERE clauaplanypais=TO_CHAR(IDAPLIC)||PAIS||EXTRACT(YEAR FROM TS) ;
/*Si existeix aleshores l'aplicaci ja s'ha descarregat en algun moment aquest mateix perode
per tant no fem res. El PRC_ESTAD_6, que s'executa abans, s'encarrega de comprovar que existeixi el registre pel perode actual.*/
RSP:='OK - n_dif_aplicacions no modificada ';

INSERT INTO log_procediments (ID_PRC,DATACRIDA,PENT1,PENT2,PENT3,PARAM_SORT) VALUES ('PRC_ESTAD_8',TS,IDAPLIC,ANYY,PAIS,RSP);
/**/
EXCEPTION
 WHEN NO_DATA_FOUND THEN
  /*Sin, voldr dir que l'aplicaci s la primera vegada que es descarrega en el perode actual
i per tant actualitzarem la taula.Tamb inserirem un registre a la taula APLIC_ANY_PAIS per tal d'informar */
  INSERT INTO APLIC_ANY_PAIS VALUES (TO_CHAR(IDAPLIC)||PAIS||EXTRACT(YEAR FROM TS));
  UPDATE estadistica_any_pais SET n_dif_aplicacions=n_dif_aplicacions+1 WHERE PERIODE=ANYY AND id_pais=PAIS;
  RSP:='OK - n_dif_aplicacions modificada UPDATE';
  INSERT INTO log_procediments (ID_PRC,DATACRIDA,PENT1,PENT2,PENT3,PARAM_SORT) VALUES ('PRC_ESTAD_8',TS,IDAPLIC,ANYY,PAIS,RSP);
  
  WHEN OTHERS THEN
   /* Informem de l'error a la variable de sortida i guardem en el log de procediments*/
  RSP:=SUBSTR(sqlerrm,1,200);
  INSERT INTO log_procediments (ID_PRC,DATACRIDA,PENT1,PENT2,PENT3,PARAM_SORT) VALUES ('PRC_ESTAD_8',TS,IDAPLIC,ANYY,PAIS,RSP);

END PRC_ESTAD_8;

